from __future__ import annotations

import base64
import hashlib
import io
import re
import time
import urllib.error
import urllib.parse
import urllib.request
from collections.abc import Callable
from dataclasses import dataclass
from pathlib import Path

from PIL import Image


# 简单的默认日志函数，便于在测试时替换
def _noop_logger(message: str) -> None:
    return None


def clear_copy_image(content: str) -> str:
    """移除尾随 `Copy Image` 的图片标记。"""
    pattern = r"!\[([^\]]*)\]\(([^\)]+)\)\s*Copy\s+Image\s*\n?"
    return re.sub(pattern, "", content, flags=re.IGNORECASE)


def sanitize_filename(url: str) -> str:
    """基于 URL 生成稳定文件名，追加短哈希并清理非法字符。"""
    parsed = urllib.parse.urlparse(url)
    name = Path(parsed.path).name
    url_hash = hashlib.md5(url.encode()).hexdigest()[:6]

    if not name or "." not in name:
        filename = f"{url_hash}.png"
    else:
        stem = Path(name).stem
        suffix = Path(name).suffix

        # 检查是否为“奇怪”的文件名
        # 1. 长度超过 50
        # 2. 包含 _3A (:) 或 _2F (/) 这种 URL 转义痕迹
        is_strange = len(stem) > 50 or "_3A" in stem or "_2F" in stem

        if is_strange:
            filename = f"img_{url_hash}{suffix}"
        else:
            filename = f"{stem}_{url_hash}{suffix}"

    return re.sub(r"[^\w\-_\.]", "_", filename)


def escape_markdown_path(path: str) -> str:
    """仅转义 Markdown 路径中的下划线。"""
    return path.replace("_", r"\_")


def encode_url_properly(url: str, log: Callable[[str], None] = _noop_logger) -> str:
    """重新编码 URL 的 path/query，保留已有的 %。"""
    try:
        parsed = urllib.parse.urlsplit(url)
        encoded_path = urllib.parse.quote(parsed.path, safe="/%:@-._~!$,")

        encoded_query = ""
        if parsed.query:
            pairs = urllib.parse.parse_qsl(parsed.query, keep_blank_values=True)
            encoded_query = "&".join(
                f"{urllib.parse.quote(k, safe='-_.~%!$,')}={urllib.parse.quote(v, safe='-_.~%!$,')}"
                for k, v in pairs
            )

        encoded_url = urllib.parse.urlunsplit(
            (parsed.scheme, parsed.netloc, encoded_path, encoded_query, parsed.fragment)
        )
        if encoded_url != url:
            log(f"   🔧 URL编码：{url[:60]}... -> {encoded_url[:60]}...")
        return encoded_url
    except Exception as exc:  # pragma: no cover - 防御性日志
        log(f"   ⚠️  URL编码失败，使用原始URL：{exc}")
        return url


def is_wechat_image(url: str) -> bool:
    return "mmbiz.qpic.cn" in url.lower() or "mp.weixin.qq.com" in url.lower()


def clean_wechat_image_url(url: str) -> str:
    params_to_remove = {"wxfrom", "wx_lazy", "tp"}
    parsed = urllib.parse.urlsplit(url)
    query_pairs = urllib.parse.parse_qsl(parsed.query, keep_blank_values=True)
    filtered = [(k, v) for k, v in query_pairs if k not in params_to_remove]
    if not any(k == "wx_fmt" for k, _ in filtered):
        filtered.append(("wx_fmt", "jpeg"))
    new_query = urllib.parse.urlencode(filtered, doseq=True)
    return urllib.parse.urlunsplit(
        (parsed.scheme, parsed.netloc, parsed.path, new_query, parsed.fragment)
    )


def get_wechat_headers(url: str) -> dict[str, str]:
    referer = "https://mp.weixin.qq.com/"
    match = re.search(r"mp\.weixin\.qq\.com/s/([a-zA-Z0-9_-]+)", url)
    if match:
        referer = f"https://mp.weixin.qq.com/s/{match.group(1)}"
    return {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
        "Accept": "image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Accept-Encoding": "gzip, deflate, br",
        "Referer": referer,
        "Sec-Fetch-Dest": "image",
        "Sec-Fetch-Mode": "no-cors",
        "Sec-Fetch-Site": "cross-site",
        "Cache-Control": "no-cache",
        "Pragma": "no-cache",
    }


def detect_image_format(data: bytes) -> str:
    if data.startswith(b"\x89PNG\r\n\x1a\n"):
        return "png"
    if data.startswith(b"\xff\xd8\xff"):
        return "jpeg"
    if data.startswith(b"GIF87a") or data.startswith(b"GIF89a"):
        return "gif"
    if data.startswith(b"RIFF") and len(data) > 12 and data[8:12] == b"WEBP":
        return "webp"
    if data.startswith(b"BM"):
        return "bmp"
    if data.startswith(b"\x00\x00\x01\x00"):
        return "ico"
    if b"<svg" in data[:1024] or data.startswith(b"<?xml"):
        return "svg"
    return "png"


def is_data_uri(url: str) -> bool:
    return url.strip().lower().startswith("data:image/")


def find_existing_image(base_path: Path) -> Path | None:
    for ext in ["png", "jpeg", "jpg", "gif", "webp", "bmp", "ico", "svg"]:
        candidate = Path(f"{base_path}.{ext}")
        if candidate.exists():
            return candidate
    return None


def _default_downloader(url: str, headers: dict[str, str]) -> bytes:
    request = urllib.request.Request(url, headers=headers)
    with urllib.request.urlopen(request, timeout=30) as response:
        return response.read()


def _convert_webp_to_png(data: bytes, log: Callable[[str], None]) -> bytes | None:
    try:
        image = Image.open(io.BytesIO(data))
        buffer = io.BytesIO()
        image.save(buffer, format="PNG")
        return buffer.getvalue()
    except Exception as exc:
        log(f"   ⚠️  WebP 转换失败，保留原格式：{exc}")
        return None


@dataclass
class DownloadResult:
    ok: bool
    path: Path | None
    fmt: str = ""
    error: str | None = None
    is_wechat: bool = False


def save_image_bytes(
    data: bytes, base_path: Path, log: Callable[[str], None] = _noop_logger
) -> tuple[Path, str]:
    fmt = detect_image_format(data)
    if fmt == "webp":
        converted = _convert_webp_to_png(data, log)
        if converted is not None:
            data = converted
            fmt = "png"
    save_path = Path(f"{base_path}.{fmt}")
    save_path.parent.mkdir(parents=True, exist_ok=True)
    with open(save_path, "wb") as fh:
        fh.write(data)
    return save_path, fmt


def save_data_uri_image(
    data_uri: str, base_path: Path, log: Callable[[str], None] = _noop_logger
) -> DownloadResult:
    try:
        match = re.match(r"data:image/([^;]+);base64,(.+)", data_uri.strip(), re.DOTALL)
        if not match:
            return DownloadResult(False, None, error="data uri parse failed")
        encoded = match.group(2)
        try:
            data = base64.b64decode(encoded)
        except Exception as exc:
            log(f"   ⚠️  Base64 解码失败：{exc}")
            return DownloadResult(False, None, error="base64 decode failed")
        path, fmt = save_image_bytes(data, base_path, log=log)
        return DownloadResult(True, path, fmt=fmt)
    except Exception as exc:  # pragma: no cover - 防御性
        log(f"   ❌ 保存 data URI 图片失败：{exc}")
        return DownloadResult(False, None, error=str(exc))


def download_image(
    url: str,
    base_path: Path,
    *,
    downloader: Callable[[str, dict[str, str]], bytes] | None = None,
    max_retries: int = 3,
    retry_delay: float = 2.0,
    log: Callable[[str], None] = _noop_logger,
) -> DownloadResult:
    downloader_func = downloader or _default_downloader
    encoded_url = encode_url_properly(url, log=log)
    is_wechat = is_wechat_image(encoded_url)

    if is_wechat:
        download_url = clean_wechat_image_url(encoded_url)
        headers = get_wechat_headers(encoded_url)
        log(f"   🔧 微信图片特殊处理：{download_url[:50]}...")
    else:
        download_url = encoded_url
        headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            "Accept": "image/*,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        }

    last_error: str | None = None

    for attempt in range(max_retries):
        try:
            data = downloader_func(download_url, headers)
            path, fmt = save_image_bytes(data, base_path, log=log)
            return DownloadResult(True, path, fmt=fmt, is_wechat=is_wechat)
        except urllib.error.HTTPError as exc:
            last_error = f"HTTP {exc.code}"
            log(f"   🔍 HTTP错误详情：状态码={exc.code}, 原因={getattr(exc, 'reason', 'unknown')}")
            if exc.code == 429 and attempt < max_retries - 1:
                wait_time = retry_delay * (attempt + 1)
                log(
                    f"⚠️  请求过于频繁 (429)，等待 {wait_time:.1f} 秒后重试... (尝试 {attempt + 1}/{max_retries})"
                )
                time.sleep(wait_time)
                continue
            if exc.code == 403 and is_wechat and attempt < max_retries - 1:
                log("⚠️  微信图片访问被拒绝 (403)，尝试移除 wxfrom 参数重试...")
                stripped = (
                    download_url.replace("&wxfrom=5", "")
                    .replace("wxfrom=5&", "")
                    .replace("?wxfrom=5", "")
                )
                download_url = stripped
                time.sleep(retry_delay)
                continue
            return DownloadResult(False, None, fmt="", error=last_error, is_wechat=is_wechat)
        except Exception as exc:
            last_error = str(exc)
            if attempt < max_retries - 1:
                wait_time = retry_delay * (attempt + 1)
                log(
                    f"⚠️  下载出错，等待 {wait_time:.1f} 秒后重试... (尝试 {attempt + 1}/{max_retries})"
                )
                time.sleep(wait_time)
                continue
            return DownloadResult(False, None, fmt="", error=last_error, is_wechat=is_wechat)

    return DownloadResult(False, None, fmt="", error=last_error, is_wechat=is_wechat)


def create_wechat_placeholder(alt_text: str, img_url: str) -> str:
    img_index_match = re.search(r"#imgIndex=(\d+)", img_url)
    img_index = img_index_match.group(1) if img_index_match else "?"
    label = alt_text if alt_text and alt_text != "图片" else "图片内容"
    return f"""
<div class="wechat-image-placeholder" style="
    border: 2px dashed #ccc; 
    padding: 20px; 
    margin: 10px 0; 
    text-align: center; 
    background-color: #f9f9f9;
    border-radius: 8px;
">
    <p style="color: #666; margin: 0;">📷 微信图片 #{img_index}</p>
    <p style="color: #999; font-size: 12px; margin: 5px 0 0 0;">
        原始链接: <a href="{img_url}" target="_blank">点击查看</a>
    </p>
    <p style="color: #999; font-size: 12px; margin: 0;">
        {label}
    </p>
</div>
"""



"""
init_workspace 简化模式测试
"""

import pytest

from mark2pdf.config import CONFIG_FILENAME
from mark2pdf.config.workspace import init_workspace


def test_simple_init_only_writes_config_and_frontmatter(tmp_path):
    """simple 模式仅写入配置与 frontmatter"""
    target = tmp_path / "workspace"
    target.mkdir()
    (target / "existing.txt").write_text("keep")

    init_workspace(target, simple=True)

    assert (target / CONFIG_FILENAME).exists()
    assert (target / "frontmatter.yaml").exists()

    assert not (target / "createpdf.py").exists()
    assert not (target / "in").exists()
    assert not (target / "out").exists()
    assert not (target / "tmp").exists()
    assert not (target / "template").exists()
    assert not (target / "fonts").exists()


def test_simple_init_rejects_template_copy(tmp_path):
    """simple 模式不支持模板复制"""
    target = tmp_path / "workspace"
    target.mkdir()

    with pytest.raises(ValueError, match="simple 模式不支持复制模板"):
        init_workspace(target, template_name="nb.typ", simple=True)



"""
mark2pdf.core 核心 API

提供可编程调用的转换函数，不依赖 CLI。
"""

from collections.abc import Callable
from pathlib import Path

from mark2pdf.helper_interfile.interfile_manager import (
    cleanup_sandbox,
    create_sandbox,
    link_images_dir,
)
from mark2pdf.helper_markdown import extract_frontmatter
from mark2pdf.helper_typst import check_pandoc_typst, run_pandoc_typst
from mark2pdf.helper_workingpath import (
    get_project_root,
    resolve_inout_paths,
    resolve_template_path,
    safesave_path,
)

from .options import ConversionOptions
from .postprocess import ensure_tc_postprocess
from .utils import (
    apply_default_preprocessing,
    get_output_filename,
    inject_default_frontmatter,
)


def _collect_font_paths(config: object | None, template_path: str | None) -> list[str]:
    font_paths: list[Path] = []
    if config is not None:
        try:
            fonts_dir = getattr(config, "fonts_dir", None)
        except Exception:
            fonts_dir = None
        if fonts_dir:
            font_paths.append(Path(fonts_dir))
    else:
        font_paths.append(Path.cwd() / "fonts")

    if template_path:
        font_paths.append(Path(template_path).parent / "fonts")

    return [str(path) for path in font_paths]


_FONT_EXTENSIONS = (".ttf", ".otf", ".ttc", ".otc")
_FONT_NAME_CACHE: dict[str, tuple[float, tuple[str, ...]]] = {}


def _get_cached_font_stems(font_dir: Path) -> tuple[str, ...]:
    try:
        stat = font_dir.stat()
    except FileNotFoundError:
        return ()
    cache_key = str(font_dir.resolve())
    cached = _FONT_NAME_CACHE.get(cache_key)
    if cached and cached[0] == stat.st_mtime:
        return cached[1]

    stems = []
    for font_file in font_dir.rglob("*"):
        if not font_file.is_file():
            continue
        if font_file.suffix.lower() not in _FONT_EXTENSIONS:
            continue
        stems.append(font_file.stem.lower())

    stems_tuple = tuple(stems)
    _FONT_NAME_CACHE[cache_key] = (stat.st_mtime, stems_tuple)
    return stems_tuple


def _font_name_in_dirs(font_name: str, font_paths: list[str]) -> bool:
    needle = font_name.strip().lower()
    if not needle:
        return True
    for font_dir in font_paths:
        dir_path = Path(font_dir)
        if not dir_path.exists() or not dir_path.is_dir():
            continue
        for stem in _get_cached_font_stems(dir_path):
            if needle in stem:
                return True
    return False


def _warn_missing_fonts(
    content: str,
    font_paths: list[str],
    verbose: bool,
    frontmatter: dict | None = None,
) -> None:
    if not verbose:
        return
    if not font_paths:
        return
    if frontmatter is None:
        frontmatter = extract_frontmatter(content)
    if not isinstance(frontmatter, dict):
        frontmatter = {}

    # search in theme group first
    theme = frontmatter.get("theme")
    if isinstance(theme, dict):
        for key in ("font", "titlefont"):
            font_value = theme.get(key)
            if not isinstance(font_value, str):
                continue
            font_name = font_value.strip()
            if not font_name:
                continue
            if _font_name_in_dirs(font_name, font_paths):
                continue
            print(
                f"  ⚠️ 未在字体目录中找到字体 '{font_name}' (theme.{key})，若系统未安装将回退默认字体"
            )


def execute_in_sandbox(
    content: str,
    temp_filename: str,
    output_path: Path,
    images_source_dir: Path,
    tmp_dir: Path,
    sandbox_prefix: str,
    options: ConversionOptions,
    config: object | None = None,
    frontmatter: dict | None = None,
) -> Path | None:
    """
    在沙箱中执行 Pandoc 转换

    流程：
    1. 创建沙箱目录
    2. 链接 images 目录
    3. 写入临时 MD 文件
    4. 调用 Pandoc 转换
    5. 清理沙箱（除非 savemd=True）

    参数：
        content: 预处理后的 Markdown 内容
        temp_filename: 临时文件名（如 "test.md"）
        output_path: 输出文件路径
        images_source_dir: images 目录的来源目录
        tmp_dir: 临时目录路径
        sandbox_prefix: 沙箱目录前缀
        options: 转换配置选项
        config: PdfworkConfig 配置对象（可选）
        frontmatter: 预先解析的 frontmatter（可选，用于复用解析结果）

    返回：
        成功返回输出文件路径 Path，失败返回 None
    """
    sandbox = create_sandbox(tmp_dir, prefix=sandbox_prefix, verbose=options.verbose)

    # 链接 images 目录
    link_images_dir(sandbox, images_source_dir, verbose=options.verbose)

    # 写入临时文件
    temp_path = sandbox / temp_filename
    temp_path.write_text(content, encoding="utf-8")
    if options.verbose:
        print(f"  ✓ 创建临时文件：{temp_path}")

    # 解析模板路径
    template_path = resolve_template_path(options.template, config=config)
    font_paths = _collect_font_paths(config, template_path)
    _warn_missing_fonts(content, font_paths, options.verbose, frontmatter=frontmatter)

    # Pandoc 参数
    pandoc_args = {}
    if options.coverimg:
        pandoc_args["coverimg"] = options.coverimg

    # 构建 disables 列表
    disables = []
    if options.no_cover:
        disables.append("cover")
    if options.no_toc:
        disables.append("toc")
    if disables:
        pandoc_args["disables"] = disables

    success = False  # 在 try 块之前初始化，防止异常时 finally 中 UnboundLocalError
    try:
        success = run_pandoc_typst(
            input_file=temp_filename,
            output_file=output_path,
            template_path=template_path,
            pandoc_workdir=sandbox,
            verbose=options.verbose,
            to_typst=options.to_typst,
            font_paths=font_paths,
            **pandoc_args,
        )
    finally:
        if options.savemd:
            if options.verbose or success:
                print(f"预处理后的 Markdown 已保存至：{temp_path}")
        else:
            cleanup_sandbox(sandbox, verbose=options.verbose)

    if not success:
        print("转换失败")
        return None

    return output_path


def convert_file(
    input_file: str,
    output_file: str | None = None,
    options: ConversionOptions = ConversionOptions(),
    indir: str | None = None,
    outdir: str | None = None,
    default_frontmatter: dict | None = None,
    postprocess: Callable[[str], str] | None = None,
    config: object | None = None,
) -> Path | None:
    """
    转换单个 Markdown 文件为 PDF

    参数：
        input_file: 输入 Markdown 文件名（相对于 indir）
        output_file: 输出文件名（可选，不含路径）
        options: 转换配置选项
        indir: 输入目录（默认从配置读取）
        outdir: 输出目录（默认从配置读取）
        default_frontmatter: 默认 frontmatter 字典，用于注入缺失的字段
        postprocess: 后处理函数，接收内容返回处理后内容。在内置预处理之后追加执行
        config: PdfworkConfig 配置对象（可选）

    返回：
        成功返回输出文件路径 Path，失败返回 None
    """
    check_pandoc_typst()

    ext = "typ" if options.to_typst else "pdf"

    if config is None:
        try:
            from mark2pdf import ConfigManager

            config = ConfigManager.load()
        except ImportError:
            config = None

    # 获取输入路径
    input_path, output_path = resolve_inout_paths(
        infile=input_file,
        outfile=output_file,
        indir=indir,
        outdir=outdir,
        ext=ext,
        config=config,
    )

    # 如果 overwrite 为 True，重新计算输出路径（不添加时间戳）
    if options.overwrite:
        outdir_path = Path(output_path).parent
        input_path_obj = Path(input_file)
        if output_file:
            output_path = outdir_path / f"{Path(output_file).stem}.{ext}"
        else:
            output_path = outdir_path / f"{input_path_obj.stem}.{ext}"
    else:
        # 确保 output_path 为 Path 类型
        output_path = Path(output_path)

    input_path = Path(input_path)

    with open(input_path, encoding="utf-8") as f:
        content = f.read()

    # 系统级：注入默认 frontmatter（在所有预处理之前）
    if default_frontmatter:
        content = inject_default_frontmatter(content, default_frontmatter, verbose=options.verbose)

    frontmatter = extract_frontmatter(content)

    # 获取最终输出文件名（可能从 exportFilename/title 覆盖）
    output_path = get_output_filename(
        content,
        output_path,
        options.tc,
        options.verbose,
        options.force_filename,
        frontmatter=frontmatter,
    )

    # 预处理（链式执行：默认预处理 + 自定义后处理）
    if options.verbose:
        print("应用 Markdown 预处理...")

    # 1. 默认预处理（始终执行）
    content = apply_default_preprocessing(
        content, removelink=options.removelink, verbose=options.verbose
    )

    # 2. 后处理（如果有，在内置预处理之后追加执行）
    if options.tc:
        postprocess = ensure_tc_postprocess(postprocess)

    if postprocess:
        content = postprocess(content)

    # 创建沙箱并执行转换
    # 获取 tmp_dir 和 images_source_dir
    if config is not None and config.data_root is not None:
        tmp_dir = config.tmp_dir
    else:
        root_dir = get_project_root()
        tmp_dir = root_dir / "tmp"
    indir_path = input_path.parent

    from mark2pdf.core.compress import compress_pdf
    
    result = execute_in_sandbox(
        content=content,
        temp_filename=input_path.name,
        output_path=output_path,
        images_source_dir=indir_path,
        tmp_dir=tmp_dir,
        sandbox_prefix="md2pdf_",
        options=options,
        config=config,
        frontmatter=frontmatter,
    )

    if result and options.compress and ext == "pdf":
        if options.verbose:
            print("正在压缩 PDF...")
        # 压缩时使用临时文件
        temp_comp = result.with_suffix(".comp.pdf")
        compress_pdf(result, temp_comp, verbose=options.verbose)
        temp_comp.replace(result)
        
    return result


def convert_from_string(
    content: str,
    output_path: Path | str,
    options: ConversionOptions = ConversionOptions(),
    default_frontmatter: dict | None = None,
    postprocess: Callable[[str], str] | None = None,
    images_dir: Path | str | None = None,
    config: object | None = None,
) -> Path | None:
    """
    从内存字符串转换为 PDF/Typst 文件

    参数：
        content: Markdown 内容
        output_path: 输出文件路径（可为 PDF 或 Typst）
        options: 转换配置选项
        default_frontmatter: 默认 frontmatter 字典，用于注入缺失的字段
        postprocess: 后处理函数，接收内容返回处理后内容。在内置预处理之后追加执行
        images_dir: 图片目录（可选，images/ 的父目录或 images/ 本身）
        config: PdfworkConfig 配置对象（可选）

    返回：
        成功返回输出文件路径 Path，失败返回 None
    """
    check_pandoc_typst()

    ext = "typ" if options.to_typst else "pdf"

    if config is None:
        try:
            from mark2pdf import ConfigManager

            config = ConfigManager.load()
        except ImportError:
            config = None

    output_path = Path(output_path).expanduser()
    if not output_path.suffix:
        output_path = output_path.with_suffix(f".{ext}")
    if not output_path.is_absolute():
        output_path = output_path.absolute()

    if not options.overwrite:
        output_path = Path(safesave_path(output_path))

    output_path.parent.mkdir(parents=True, exist_ok=True)

    # 系统级：注入默认 frontmatter（在所有预处理之前）
    if default_frontmatter:
        content = inject_default_frontmatter(content, default_frontmatter, verbose=options.verbose)

    frontmatter = extract_frontmatter(content)

    # 获取最终输出文件名（可能从 exportFilename/title 覆盖）
    output_path = get_output_filename(
        content,
        output_path,
        options.tc,
        options.verbose,
        options.force_filename,
        frontmatter=frontmatter,
    )

    # 预处理（链式执行：默认预处理 + 自定义后处理）
    if options.verbose:
        print("应用 Markdown 预处理...")

    # 1. 默认预处理（始终执行）
    content = apply_default_preprocessing(
        content, removelink=options.removelink, verbose=options.verbose
    )

    # 2. 后处理（如果有，在内置预处理之后追加执行）
    if options.tc:
        postprocess = ensure_tc_postprocess(postprocess)

    if postprocess:
        content = postprocess(content)

    # 获取 tmp_dir
    if config is not None and config.data_root is not None:
        tmp_dir = config.tmp_dir
    else:
        root_dir = get_project_root()
        tmp_dir = root_dir / "tmp"

    if images_dir:
        images_path = Path(images_dir)
        images_source_dir = images_path.parent if images_path.name == "images" else images_path
    else:
        images_source_dir = tmp_dir

    return execute_in_sandbox(
        content=content,
        temp_filename="input.md",
        output_path=output_path,
        images_source_dir=images_source_dir,
        tmp_dir=tmp_dir,
        sandbox_prefix="str_",
        options=options,
        config=config,
        frontmatter=frontmatter,
    )



#!/usr/bin/env python3

"""
mark2pdf.helper_interfile 使用示例
"""

from pathlib import Path

from mark2pdf.helper_interfile.interfile_manager import (
    cleanup_tmpfile,
    create_md_tmpfile,
    write_to_tmpfile,
)


def example_usage():
    """示例用法"""

    # 创建一个测试文件
    test_file = "test_input.md"
    with open(test_file, "w", encoding="utf-8") as f:
        f.write("# Original Content\n\nThis is the original markdown file.")

    print(f"📄 创建测试文件：{test_file}")

    try:
        # 1. 创建临时文件
        temp_fd, temp_path = create_md_tmpfile(test_file, prefix="example_")
        print(f"✅ 创建临时文件：{temp_path}")

        # 2. 写入处理后的内容
        processed_content = "# Processed Content\n\nThis is processed markdown content."
        write_to_tmpfile(temp_fd, processed_content)
        print("✅ 成功写入临时文件")

        # 验证写入的内容
        with open(temp_path, encoding="utf-8") as f:
            content = f.read()
        print(f"📝 临时文件内容:\n{content}")

    finally:
        # 3. 清理临时文件
        if "temp_path" in locals():
            cleanup_tmpfile(temp_path)
            print("✅ 成功清理临时文件")

        # 清理测试文件
        if Path(test_file).exists():
            Path(test_file).unlink()
            print("✅ 清理测试文件")


if __name__ == "__main__":
    example_usage()



"""
mark2pdf 版本命令
"""

import click


def get_version() -> str:
    """获取包版本号"""
    try:
        from importlib.metadata import version

        return version("yakultpdf")
    except Exception:
        return "unknown"


@click.command()
def version():
    """显示版本信息"""
    click.echo(f"mark2pdf {get_version()}")



"""
mark2pdf CLI 测试
"""

import sys
from pathlib import Path
from unittest.mock import patch

from click.testing import CliRunner

# 添加父目录路径
sys.path.insert(0, str(Path(__file__).parent.parent.parent))

from mark2pdf.cli import main


class TestInitCommand:
    """init 命令测试"""

    def test_init_missing_arg(self):
        """测试缺少参数"""
        runner = CliRunner()
        result = runner.invoke(main, ["init"])
        assert result.exit_code != 0
        assert "Missing argument 'TARGET_DIR'" in result.output

    def test_init_non_empty_dir_fails(self, tmp_path):
        """测试在非空目录运行失败"""
        # 创建一个非空目录
        target = tmp_path / "non_empty"
        target.mkdir()
        (target / "foo.txt").touch()

        runner = CliRunner()
        # 必须传入目录参数
        result = runner.invoke(main, ["init", str(target)])

        assert result.exit_code != 0
        assert "目录不为空" in result.output
        assert "禁止在非空目录初始化" in result.output

    def test_init_empty_dir_succeeds(self, tmp_path):
        """测试在空目录运行成功"""
        target = tmp_path / "empty"
        target.mkdir()

        runner = CliRunner()
        with patch("mark2pdf.commands.workspace.init_workspace") as mock_init:
            result = runner.invoke(main, ["init", str(target)])

            assert result.exit_code == 0
            mock_init.assert_called_once_with(
                target.resolve(),
                template_name=None,
                simple=False,
            )

    def test_init_template_only(self, tmp_path):
        """测试 --template 仅复制指定模板"""
        target = tmp_path / "template_only"
        target.mkdir()

        runner = CliRunner()
        with patch("mark2pdf.commands.workspace.init_workspace") as mock_init:
            result = runner.invoke(main, ["init", str(target), "--template", "nb.typ"])

            assert result.exit_code == 0
            mock_init.assert_called_once_with(
                target.resolve(),
                template_name="nb.typ",
                simple=False,
            )

    def test_init_ignores_ds_store(self, tmp_path):
        """测试忽略 .DS_Store 文件"""
        target = tmp_path / "ds_store_only"
        target.mkdir()
        (target / ".DS_Store").touch()

        runner = CliRunner()
        with patch("mark2pdf.commands.workspace.init_workspace") as mock_init:
            result = runner.invoke(main, ["init", str(target)])

            assert result.exit_code == 0
            mock_init.assert_called_once_with(
                target.resolve(),
                template_name=None,
                simple=False,
            )

    def test_init_simple_non_empty_dir_succeeds(self, tmp_path):
        """测试 --simple 允许非空目录"""
        target = tmp_path / "non_empty"
        target.mkdir()
        (target / "foo.txt").touch()

        runner = CliRunner()
        with patch("mark2pdf.commands.workspace.init_workspace") as mock_init:
            result = runner.invoke(main, ["init", str(target), "--simple"])

            assert result.exit_code == 0
            mock_init.assert_called_once_with(
                target.resolve(),
                template_name=None,
                simple=True,
            )


class TestVersionCommand:
    """version 命令测试"""

    def test_version_output(self):
        """测试版本输出"""
        runner = CliRunner()
        result = runner.invoke(main, ["version"])
        assert result.exit_code == 0
        assert "mark2pdf" in result.output


class TestConvertCommand:
    """convert 命令测试"""

    def test_convert_help(self):
        """测试帮助信息"""
        runner = CliRunner()
        result = runner.invoke(main, ["convert", "--help"])
        assert result.exit_code == 0
        assert "--show-config" in result.output
        assert "--dry-run" in result.output
        assert "--postprocess" in result.output
        assert "--batchall" in result.output

    def test_convert_standalone_dry_run(self):
        """测试无配置文件时的独立模式"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("test.md").write_text("# Test")
            result = runner.invoke(main, ["convert", "test.md", "--dry-run"])
            assert result.exit_code == 0
            assert "执行计划" in result.output
            assert "paths.input: ." in result.output
            assert "paths.output: ." in result.output

    def test_convert_batchall_dry_run(self):
        """测试 --batchall 等价于批量转换当前目录"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            result = runner.invoke(main, ["convert", "--batchall", "--dry-run"])
            assert result.exit_code == 0
            # jobs=4 时显示并发模式
            assert "转换目录 '.'" in result.output

    def test_batchall_batch_conflict(self):
        """--batchall 与 --batch 不能同时使用"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            result = runner.invoke(main, ["convert", "--batchall", "--batch", "foo"])
            assert result.exit_code != 0
            assert "--batchall 与 --batch 不能同时使用" in result.output

    def test_dir_batch_conflict(self):
        """--dir 与 --batch 不能同时使用"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            # 需要跳过工具检查，否则会检查 pandoc/typst
            result = runner.invoke(
                main, ["convert", "--dir", "foo", "--batch", "bar", "--skip-tool-check"]
            )
            assert result.exit_code != 0
            assert "--dir 和 --batch" in result.output

    def test_show_config_mode(self):
        """--show-config 仅显示配置不执行"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("test.md").write_text("# Test")
            result = runner.invoke(main, ["convert", "test.md", "--show-config"])
            assert result.exit_code == 0
            assert "合并后的完整配置" in result.output
            # 不应有执行计划
            assert "执行计划" not in result.output

    @patch("mark2pdf.commands.convert.run_conversion")
    @patch("mark2pdf.commands.convert.check_pandoc_typst")
    def test_single_file_calls_run_conversion(self, mock_check, mock_run):
        """单文件模式调用 run_conversion"""
        mock_run.return_value = True
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("test.md").write_text("# Test")
            result = runner.invoke(main, ["convert", "test.md"])
            assert result.exit_code == 0
            mock_run.assert_called_once()
            # 验证调用参数
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs["filename"] == "test.md"

    @patch("mark2pdf.commands.convert.run_directory_conversion")
    @patch("mark2pdf.commands.convert.check_pandoc_typst")
    def test_dir_mode_calls_run_directory_conversion(self, mock_check, mock_run):
        """--dir 模式调用 run_directory_conversion"""
        mock_run.return_value = True
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("docs").mkdir()
            (Path("docs") / "index.md").write_text("# Docs")
            result = runner.invoke(main, ["convert", "--dir", "docs"])
            assert result.exit_code == 0
            mock_run.assert_called_once()
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs["directory"] == "docs"

    @patch("mark2pdf.commands.convert.run_batch_conversion")
    @patch("mark2pdf.commands.convert.check_pandoc_typst")
    def test_batch_mode_calls_run_batch_conversion(self, mock_check, mock_run):
        """--batch 模式调用 run_batch_conversion"""
        mock_run.return_value = True
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("batch").mkdir()
            (Path("batch") / "file1.md").write_text("# File 1")
            result = runner.invoke(main, ["convert", "--batch", "batch"])
            assert result.exit_code == 0
            mock_run.assert_called_once()
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs["directory"] == "batch"

class TestTemplateCommand:
    """template 命令测试"""

    def test_template_requires_workspace(self):
        """测试无工作区配置时失败"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            result = runner.invoke(main, ["template", "nb.typ"])
            assert result.exit_code != 0
            assert "未检测到工作区配置" in result.output

    def test_template_copies_to_workspace(self):
        """测试复制模板到工作区"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("mark2pdf.config.toml").write_text('[project]\nname = "test"')

            result = runner.invoke(main, ["template", "card.typ"])

            assert result.exit_code == 0
            assert Path("template/card.typ").exists()

    def test_template_copies_directory(self):
        """测试复制模板目录"""
        runner = CliRunner()
        with runner.isolated_filesystem():
            Path("mark2pdf.config.toml").write_text('[project]\nname = "test"')

            result = runner.invoke(main, ["template", "letter"])

            assert result.exit_code == 0
            assert Path("template/letter").exists()


class TestCoverPrepareCommand:
    """coverimg 命令测试"""

    def test_list_papers(self):
        """测试列出纸型"""
        runner = CliRunner()
        result = runner.invoke(main, ["coverimg", "list"])
        assert result.exit_code == 0
        assert "a4" in result.output
        assert "letter" in result.output

    def test_check_help(self):
        """测试 check 子命令帮助"""
        runner = CliRunner()
        result = runner.invoke(main, ["coverimg", "check", "--help"])
        assert result.exit_code == 0
        assert "--paper" in result.output



"""
稿纸专用预处理模块

提供稿纸格式（方格纸）的 Markdown 预处理函数：
- center_with_cnspace: 用中文空格居中文本
- md_pre_process: 清理 frontmatter、移除空格、转换段落格式
- process_for_typ: 稿纸专用排版（标题居中、作者居中、正文段首缩进）
"""

import re


def center_with_cnspace(text: str, linelength: int = 20) -> str:
    """在文本前添加中文空格，使其在指定长度的一行中居中

    参数：
        text: 需要居中的文本
        linelength: 行的总长度（字符数）

    返回：
        添加了中文空格前缀的居中文本
    """
    cnspace = "\u3000"  # '\u3000' 为中文空格
    length = len(text)
    spaces_needed = (linelength - length - 1) // 2
    if spaces_needed > 0 and length <= linelength:
        return cnspace * (spaces_needed + 1) + text
    else:
        return text


def md_pre_process(content: str) -> list[str]:
    """前置处理内容，包括清理 frontmatter、多余换行、转换段落格式等

    处理步骤：
    1. 清除 YAML frontmatter
    2. 移除每行前后的空格
    3. 移除所有英文空格
    4. 移除所有中文空格（稍后统一添加）
    5. 将单换行转为双换行，规范化换行
    6. 将双换行替换为反斜杠+换行（Typst 段落格式）

    参数：
        content: 原始 Markdown 内容

    返回：
        处理后的行列表
    """
    if not content.strip():
        return []

    # - 清除 frontmatter（在 split 之前处理）
    content = re.sub(r"^---\n.*?\n---\n", "", content, flags=re.DOTALL)  # re.DOTALL 用来匹配换行符

    # - 移除每行前后的空格
    content = re.sub(r"^[ \t]+|[ \t]+$", "", content, flags=re.MULTILINE)

    # - 移除所有空格（注意这个可能带来的负面影响!）
    content = re.sub(r" ", "", content)

    # - 移除所有中文空格（包括连续的），稍后会统一添加
    content = re.sub(r"\u3000+", "", content)

    # - 将所有单个换行转换为双换行，再将 3 个或更多连续换行转换为 2 个换行
    content = re.sub(r"\n", "\n\n", content)
    content = re.sub(r"\n{3,}", "\n\n", content)

    # - 去除开头和结尾的空行（使用 strip 处理所有空白字符）
    content = content.strip()

    # - 将所有的双换行（段落分隔符）替换为反斜杠 + 换行
    content = re.sub(r"\n\n", "\\\n", content)

    # - 去除开头和结尾的空行（使用 strip 处理所有空白字符）
    lines = content.strip().split("\n")

    return lines


def process_for_typ(content: str) -> str:
    """处理 markdown 内容，按照指定格式处理，便于 Typst 稿纸模板使用

    排版规则：
    1. 第一行（标题）：用中文空格居中
    2. 第二行（作者）：如果少于8个字符则居中，否则添加两个中文空格缩进
    3. 其余行（正文）：添加两个中文空格作为段首缩进

    参数：
        content: 原始 Markdown 内容

    返回：
        处理后适合稿纸模板的文本
    """
    # 前置处理
    result_lines = md_pre_process(content)

    # (A) 处理标题和作者行
    if len(result_lines) >= 1:
        # 第一行是标题，添加中文空格居中
        result_lines[0] = center_with_cnspace(result_lines[0])

    if len(result_lines) >= 2:
        # 第二行是作者，只有少于 8 个字才进行居中处理，否则视为普通行
        maybe_author_line = result_lines[1].strip()
        if len(maybe_author_line) < 8:  # "某某某\"(4)，"作者：某某某\"(7)
            result_lines[1] = center_with_cnspace(maybe_author_line)
        else:
            result_lines[1] = "\u3000\u3000" + maybe_author_line

    # (B) 为所有非空行添加中文空格前缀（除了标题和作者）
    for i, line in enumerate(result_lines):
        if i > 1 and line.strip() != "":
            result_lines[i] = "\u3000\u3000" + line

    return "\n".join(result_lines)



"""
mark2pdf 工作区管理器

负责工作区的初始化、检测与模板安装。
"""

import shutil
from contextlib import nullcontext
from importlib import resources
from pathlib import Path

from .defaults import CONFIG_FILENAME, DEFAULT_CONFIG_CONTENT
from .loader import ConfigManager, get_code_root

_IGNORE_TEMPLATE_NAMES = {".DS_Store", "__pycache__", "__init__.py"}


def _has_bundled_templates(root) -> bool:
    for item in root.iterdir():
        if item.name in _IGNORE_TEMPLATE_NAMES:
            continue
        return True
    return False


def _get_template_source_context():
    try:
        bundled_root = resources.files("mark2pdf.templates")
    except (ModuleNotFoundError, AttributeError):
        bundled_root = None

    if bundled_root is not None and _has_bundled_templates(bundled_root):
        return resources.as_file(bundled_root)

    try:
        code_root = get_code_root()
    except FileNotFoundError:
        return nullcontext(None)

    template_root = code_root / "src" / "mark2pdf" / "templates"
    if not template_root.exists():
        return nullcontext(None)
    return nullcontext(template_root)


def _copy_all_templates(source_root: Path, target_root: Path) -> None:
    target_root.mkdir(parents=True, exist_ok=True)
    ignore = shutil.ignore_patterns(*_IGNORE_TEMPLATE_NAMES)
    for item in source_root.iterdir():
        if item.name in _IGNORE_TEMPLATE_NAMES:
            continue
        target_path = target_root / item.name
        if item.is_dir():
            shutil.copytree(item, target_path, dirs_exist_ok=True, ignore=ignore)
        else:
            shutil.copy2(item, target_path)


def _copy_single_template(source_root: Path, target_root: Path, template_name: str) -> None:
    if "/" in template_name or "\\" in template_name:
        raise ValueError(f"模板文件名 '{template_name}' 包含目录路径，请输入纯文件名")

    template_path = source_root / template_name

    # 目录式模板：复制整个目录
    if template_path.exists() and template_path.is_dir():
        target_root.mkdir(parents=True, exist_ok=True)
        target_dir = target_root / template_path.name
        ignore = shutil.ignore_patterns(*_IGNORE_TEMPLATE_NAMES)
        shutil.copytree(template_path, target_dir, dirs_exist_ok=True, ignore=ignore)
        return

    # 检查是否有同名目录（不带扩展名）
    # 例如：template_name = "nb.typ"，检查是否存在 source_root/nb/ 目录
    if template_name.endswith(".typ"):
        dir_name = template_name[:-4]
        dir_path = source_root / dir_name
        if dir_path.exists() and dir_path.is_dir():
            target_root.mkdir(parents=True, exist_ok=True)
            target_dir = target_root / dir_name
            ignore = shutil.ignore_patterns(*_IGNORE_TEMPLATE_NAMES)
            shutil.copytree(dir_path, target_dir, dirs_exist_ok=True, ignore=ignore)
            return

    # 单文件模板：直接复制
    if not template_path.exists():
        raise FileNotFoundError(f"模板文件不存在：{template_path}")

    target_root.mkdir(parents=True, exist_ok=True)
    shutil.copy2(template_path, target_root / template_path.name)


def detect_workspace() -> Path | None:
    """
    检测当前目录是否为工作区

    返回工作区目录，若不是工作区则返回 None
    """
    cwd = Path.cwd()
    for parent in [cwd] + list(cwd.parents):
        if (parent / CONFIG_FILENAME).exists():
            return parent
    return None


def init_workspace(
    target_dir: Path | str | None = None,
    copy_templates: bool = False,
    template_name: str | None = None,
    simple: bool = False,
) -> Path:
    """
    初始化工作区

    创建标准目录结构、配置文件、帮助脚本和示例文件。
    simple 模式仅创建配置文件与 frontmatter 示例。

    Args:
        target_dir: 目标目录，默认为 CWD
        copy_templates: 是否复制系统模板到本地
        template_name: 仅复制指定模板文件（文件名）
        simple: 仅创建配置与 frontmatter，允许非空目录

    Returns:
        Path: 工作区根目录

    Raises:
        FileExistsError: 目录不为空
    """
    target = Path(target_dir).resolve() if target_dir else Path.cwd()

    if target.exists() and not target.is_dir():
        raise ValueError(f"目标不是目录: {target}")

    if copy_templates and template_name:
        raise ValueError("copy_templates 与 template_name 不能同时使用")

    if simple and (copy_templates or template_name):
        raise ValueError("simple 模式不支持复制模板")

    # 检查非空目录
    if not simple and target.exists() and target.is_dir():
        has_files = False
        for child in target.iterdir():
            if child.name == ".DS_Store":
                continue
            has_files = True
            break

        if has_files:
            raise FileExistsError(
                f"目录不为空: {target}\n为了安全起见，禁止在非空目录初始化。请选择空目录。"
            )
    templates_dir = Path(__file__).resolve().parents[1] / "resources"

    if simple:
        target.mkdir(parents=True, exist_ok=True)

        # 生成配置文件
        config_path = target / CONFIG_FILENAME
        if not config_path.exists():
            config_path.write_text(DEFAULT_CONFIG_CONTENT, encoding="utf-8")
            print(f"  已创建配置: {CONFIG_FILENAME}")
        else:
            print(f"  配置文件已存在: {CONFIG_FILENAME}")

        # 复制 frontmatter.yaml 示例
        fm_src = templates_dir / "frontmatter.yaml"
        fm_dst = target / "frontmatter.yaml"
        if not fm_dst.exists():
            shutil.copy(fm_src, fm_dst)
            print("  已创建配置: frontmatter.yaml")
        else:
            print("  配置已存在: frontmatter.yaml")

        print(f"\n工作区配置已初始化: {target}")
        return target

    # 创建标准目录
    for d in ["out", "tmp", "template", "fonts"]:
        dir_path = target / d
        dir_path.mkdir(parents=True, exist_ok=True)
        print(f"  已创建目录: {d}/")

    # 生成配置文件
    config_path = target / CONFIG_FILENAME
    if not config_path.exists():
        config_path.write_text(DEFAULT_CONFIG_CONTENT, encoding="utf-8")
        print(f"  已创建配置: {CONFIG_FILENAME}")
    else:
        print(f"  配置文件已存在: {CONFIG_FILENAME}")

    # 复制帮助脚本
    script_src = templates_dir / "createpdf.py"
    script_dst = target / "createpdf.py"
    if not script_dst.exists():
        shutil.copy(script_src, script_dst)
        # 添加执行权限
        script_dst.chmod(script_dst.stat().st_mode | 0o755)
        print("  已创建脚本: createpdf.py")
    else:
        print("  脚本已存在: createpdf.py")

    # 复制示例 index.md
    index_src = templates_dir / "index.md"
    index_dst = target / "index.md"
    if not index_dst.exists():
        shutil.copy(index_src, index_dst)
        print("  已创建样例: index.md")
    else:
        print("  样例已存在: index.md")

    # 复制 frontmatter.yaml 示例
    fm_src = templates_dir / "frontmatter.yaml"
    fm_dst = target / "frontmatter.yaml"
    if not fm_dst.exists():
        shutil.copy(fm_src, fm_dst)
        print("  已创建配置: frontmatter.yaml")
    else:
        print("  配置已存在: frontmatter.yaml")

    if copy_templates or template_name:
        with _get_template_source_context() as source_root:
            if source_root is None or not source_root.exists():
                raise FileNotFoundError("未找到可用的模板资源")

            target_templates = target / "template"
            if template_name:
                _copy_single_template(source_root, target_templates, template_name)
                print(f"  已复制模板: {template_name}")
            else:
                _copy_all_templates(source_root, target_templates)
                print("  已复制模板: template/")

    print(f"\n工作区已初始化: {target}")
    print("\n使用方法:")
    print(f"  uv run --directory <mark2pdf代码目录> python {target}/createpdf.py")
    return target


def install_template(template_name: str, target_dir: Path | str | None = None) -> Path:
    """
    将指定模板复制到工作区的模板目录

    Args:
        template_name: 模板文件名（如 nb.typ）
        target_dir: 目标工作区目录（可选）

    Returns:
        Path: 模板目录路径

    Raises:
        FileNotFoundError: 未找到工作区或模板资源
        ValueError: 模板文件名不合法
    """
    config = ConfigManager.load(target_dir)
    if config.standalone or config.data_root is None:
        raise FileNotFoundError("未检测到工作区配置，请先使用 'mark2pdf init <dir>' 初始化。")

    target_templates = config.template_dir
    with _get_template_source_context() as source_root:
        if source_root is None or not source_root.exists():
            raise FileNotFoundError("未找到可用的模板资源")

        _copy_single_template(source_root, target_templates, template_name)

    return target_templates



"""
测试 pre_remove_titlestar 功能
去掉标题中的加粗标记
"""

import sys
from pathlib import Path

# 添加父目录到 sys.path，使得可以导入 md_preprocess 模块
sys.path.insert(0, str(Path(__file__).parent.parent))

from md_preprocess import pre_remove_titlestar


def test_h2_with_bold():
    """测试 H2 标题中的加粗应该被去掉"""
    content = "## **Premise**"
    expected = "## Premise"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_h6_with_bold_and_colon():
    """测试 H6 标题中的加粗应该被去掉（包含冒号）"""
    content = "###### **Revenue Mechanics:**"
    expected = "###### Revenue Mechanics:"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_h1_with_bold():
    """测试 H1 标题中的加粗应该被去掉"""
    content = "# **Introduction**"
    expected = "# Introduction"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_multiple_headings():
    """测试多个标题"""
    content = """# **Title**

Some content here.

## **Subtitle**

More content.

### **Section**"""
    expected = """# Title

Some content here.

## Subtitle

More content.

### Section"""
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_heading_without_bold():
    """测试没有加粗的标题不应该被改变"""
    content = "## Regular Heading"
    expected = "## Regular Heading"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_partial_bold_in_heading():
    """测试部分加粗的标题只去掉加粗标记"""
    content = "## This is **bold** text"
    expected = "## This is bold text"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_bold_in_regular_text():
    """测试普通文本中的加粗不应该被改变"""
    content = """## **Heading**

This is **bold** text in a paragraph."""
    expected = """## Heading

This is **bold** text in a paragraph."""
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_heading_at_start_of_file():
    """测试文件开头的标题"""
    content = "# **First Heading**\n\nContent"
    expected = "# First Heading\n\nContent"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"


def test_multiple_bold_in_heading():
    """测试标题中有多个加粗部分"""
    content = "## **Part 1** and **Part 2**"
    expected = "## Part 1 and Part 2"
    result = pre_remove_titlestar(content)
    assert result == expected, f"Expected: {expected}\nGot: {result}"



"""
后处理器加载器

提供 load_postprocessor 函数，先从工作区 postprocess 目录加载，再回退到预设处理器。
"""

from collections.abc import Callable
from importlib import util
from pathlib import Path


def _load_from_workspace(name: str) -> Callable[[str], str] | None:
    """从工作区 postprocess 目录加载处理器"""
    workspace_postprocess = Path.cwd() / "postprocess"
    if not workspace_postprocess.is_dir():
        return None

    # 尝试从 __init__.py 导入
    init_file = workspace_postprocess / "__init__.py"
    if init_file.exists():
        spec = util.spec_from_file_location("_workspace_postprocess", init_file)
        if spec and spec.loader:
            module = util.module_from_spec(spec)
            spec.loader.exec_module(module)
            processor = getattr(module, name, None)
            if callable(processor):
                return processor

    # 尝试从单独的模块文件导入
    module_file = workspace_postprocess / f"{name}.py"
    if module_file.exists():
        spec = util.spec_from_file_location(f"_workspace_{name}", module_file)
        if spec and spec.loader:
            module = util.module_from_spec(spec)
            spec.loader.exec_module(module)
            processor = getattr(module, "process", None)
            if callable(processor):
                return processor

    return None


def load_postprocessor(name: str) -> Callable[[str], str] | None:
    """
    加载后处理器

    优先从工作区 postprocess 目录加载，再回退到预设处理器包。

    Args:
        name: 处理器名称（如 "remove_links", "to_traditional_chinese", "convert_divs"）

    Returns:
        处理器函数，或 None（未找到）
    """
    # 1. 先从工作区加载
    processor = _load_from_workspace(name)
    if processor:
        return processor

    # 2. 回退到预设 postprocess 包
    try:
        from mark2pdf import postprocess as builtin_postprocess

        processor = getattr(builtin_postprocess, name, None)
        if callable(processor):
            return processor
    except ImportError:
        pass

    return None


def ensure_tc_postprocess(
    postprocess: Callable[[str], str] | None,
) -> Callable[[str], str]:
    """
    确保后处理链包含繁体转换。

    如果已有处理器链包含繁体转换，则直接返回原处理器。
    """
    from mark2pdf.postprocess.to_traditional_chinese import process as tc_process

    if postprocess is None:
        return tc_process
    if postprocess is tc_process:
        return postprocess
    if getattr(postprocess, "_mark2pdf_handles_tc", False):
        return postprocess

    def combined(content: str) -> str:
        content = postprocess(content)
        return tc_process(content)

    combined._mark2pdf_handles_tc = True  # type: ignore[attr-defined]
    return combined



"""
mark2pdf 后处理器构建器

负责动态加载和组装后处理器链。
"""

from collections.abc import Callable
from dataclasses import dataclass

from mark2pdf.core.postprocess import load_postprocessor


@dataclass(frozen=True)
class PostprocessBuildResult:
    chain: Callable[[str], str] | None
    loaded: list[str]
    missing: list[str]


def build_postprocessor_chain(
    processor_names: list[str],
) -> PostprocessBuildResult:
    """
    构建后处理器链

    根据处理器名称列表动态加载并组装成一个组合函数。

    Args:
        processor_names: 处理器名称列表，如 ["remove_links", "to_traditional_chinese"]

    Returns:
        PostprocessBuildResult（包含处理链、已加载列表和缺失列表）
    """
    if not processor_names:
        return PostprocessBuildResult(chain=None, loaded=[], missing=[])

    resolved = [(p, load_postprocessor(p)) for p in processor_names]
    funcs = [f for _, f in resolved if f]  # 过滤 None

    # 检查未找到的处理器
    loaded = [p for p, f in resolved if f]
    missing = [p for p, f in resolved if f is None]

    if not funcs:
        return PostprocessBuildResult(chain=None, loaded=loaded, missing=missing)

    def combined(content: str) -> str:
        for fn in funcs:
            content = fn(content)
        return content

    if any(p == "to_traditional_chinese" and f for p, f in resolved):
        combined._mark2pdf_handles_tc = True  # type: ignore[attr-defined]

    return PostprocessBuildResult(chain=combined, loaded=loaded, missing=missing)
